﻿using Eclr;
using Eclr.Pcos;
using RTC_APILib;
using RTC_CAN_Lib;
using RTC_COMLib;
using RTC_ECAT_Lib;
using RTC_LOGLib;
using RTC_SOCKLib;
using System;
using System.Iec61131Lib;

namespace RTC_Template
{
    [FUNCTION("INT")]
    public class RTC_Program_Main
    {
        /// <summary>
        /// イベントタスク数(Express=9 / Pro+=14)
        /// </summary>
        const int EVENT_TASK_COUNT = 9;
        /// <summary>
        /// イベントタスク実行周期の上限
        /// </summary>
        const int MAX_EVENT_CYCLE = int.MaxValue;
        /// <summary>
        /// イベントタスク無効設定
        /// </summary>
        const int TASK_DISABLE = -1;
        /// <summary>
        /// イベントタスクに発行するイベント番号
        /// </summary>
        enum EN_EVENT_NO
        {
            /// <summary>
            /// Event Task 01
            /// </summary>
            Task01 = 0,
            /// <summary>
            /// Event Task 02
            /// </summary>
            Task02,
            /// <summary>
            /// Event Task 03
            /// </summary>
            Task03,
            /// <summary>
            /// Event Task 04
            /// </summary>
            Task04,
            /// <summary>
            /// Event Task 05
            /// </summary>
            Task05,
            /// <summary>
            /// Event Task 06
            /// </summary>
            Task06,
            /// <summary>
            /// Event Task 07
            /// </summary>
            Task07,
            /// <summary>
            /// Event Task 08
            /// </summary>
            Task08,
            /// <summary>
            /// Event Task 09
            /// </summary>
            Task09,
        }

        /// <summary>
        /// イベント実行周期
        /// </summary>
        static int[] minEventTime = new int[EVENT_TASK_COUNT];
        /// <summary>
        /// 周期カウンター
        /// </summary>
        static int[] minCycleCount = new int[EVENT_TASK_COUNT];

        /// <summary>
        /// コンストラクタ処理
        /// </summary>
        static RTC_Program_Main()
        {
            // 各タスクの実行周期を初期化する
            for (int i = 0; i < minEventTime.Length; i++)
            {
                fnCreateEventTime((EN_EVENT_NO)i, TASK_DISABLE);
            }
        }

        /// <summary>
        /// イベント実行周期設定
        /// </summary>
        /// <param name="enEventNo">イベント周期を設定するタスクのイベント番号</param>
        /// <param name="inSetEventTime">イベント実行周期</param>
        static void fnCreateEventTime(EN_EVENT_NO enEventNo, int inSetEventTime)
        {
            int inEventNo = (int)enEventNo;

            if (inEventNo > minEventTime.Length) { return; }          // 指定されたイベント番号がイベント実行周期の配列よりも大きい場合は処理を抜ける
            if (inSetEventTime > MAX_EVENT_CYCLE) { return; }          // 指定されたイベント実行周期が上限を超える場合はパラメータ異常と判断して処理を抜ける

            minEventTime[inEventNo] = inSetEventTime;                   // 指定されたイベントタスクのイベント実行周期を更新する
        }

        /// <summary>
        /// イベントタスクへのイベント発行処理
        /// </summary>
        /// <param name="enEventNo"></param>
        static void fnRunEvent(EN_EVENT_NO enEventNo)
        {
            int inEventNo = (int)enEventNo;

            if (minEventTime[inEventNo] == TASK_DISABLE) { return; }    // タスクが無効のものは処理しない

            minCycleCount[inEventNo]++;                                 // タスクイベント通知用のカウンタをインクリメントする
            // カウンタの値がイベント実行周期よりも大きくなった場合
            if (minCycleCount[inEventNo] >= minEventTime[inEventNo])
            {
                Resource.RaiseTaskEvent((ushort)enEventNo);             // イベント通知を発行する
                minCycleCount[inEventNo] = 0;                           // カウンタを初期化する
            }
        }

        /// <summary>
        /// 初期化時に行う処理
        /// </summary>
        /// <returns></returns>
        static short On_Init()
        {
            Int16 inReturn = CRTC.STS_INI;

            //---------------------------------------------------------
            //↓以下、ユーザが初期処理を記述します。
            //---------------------------------------------------------

            inReturn = CRTC.STS_PRO;

            //---------------------------------------------------------
            //↑ユーザが記述する初期処理はここまでです。
            //---------------------------------------------------------

            return inReturn;
        }

        //-----------------------------------
        // ■通常処理
        //-----------------------------------
        static short On_Process()
        {
            Int16 inReturn = CRTC.STS_PRO;

            //---------------------------------------------------------
            //↓以下、ユーザが通常処理を記述します。
            //---------------------------------------------------------

            //---------------------------------------------------------
            //↑ユーザが記述する通常処理はここまでです。
            //---------------------------------------------------------

            return inReturn;
        }


        //-----------------------------------
        // ■終了処理
        //-----------------------------------
        static short On_End()
        {
            Int16 inReturn = CRTC.STS_END;

            //---------------------------------------------------------
            //↓以下、ユーザが初期処理を記述します。
            //---------------------------------------------------------

            //---------------------------------------------------------
            //↑ユーザが記述する初期処理はここまでです。
            //---------------------------------------------------------

            return inReturn;
        }


        //-----------------------------------
        // ■エラー処理
        //-----------------------------------
        static short On_Err()
        {
            Int16 inReturn = CRTC.STS_ERR;

            //---------------------------------------------------------
            //↓以下、ユーザがエラー処理を記述します。
            //---------------------------------------------------------

            //---------------------------------------------------------
            //↑ユーザが記述するエラー処理はここまでです。
            //---------------------------------------------------------

            return inReturn;
        }

        /// <summary>
        /// 他のタスクを起動する優先度"0"で動作するプログラム(優先度高0～優先度低15)
        /// </summary>
        /// <param name="inState"></param>
        /// <returns></returns>
        public static Int16 __Process([VAR_INPUT("INT")] Int16 inState)
        {
            Int16 inReturn = 99; // 終了時のステータス格納用変数
            unsafe
            {
                CRTC.pMArea->inSystemCounter += 1;      // システムカウンターをインクリメントする
            }

            switch (inState)
            {
                case CRTC.STS_INI:
                    // タスクのイベント実行周期設定
                    fnCreateEventTime(EN_EVENT_NO.Task01, 1);
                    fnCreateEventTime(EN_EVENT_NO.Task02, 1);
                    fnCreateEventTime(EN_EVENT_NO.Task03, 1);
                    fnCreateEventTime(EN_EVENT_NO.Task04, 1);
                    fnCreateEventTime(EN_EVENT_NO.Task05, 1);
                    fnCreateEventTime(EN_EVENT_NO.Task06, 1);
                    fnCreateEventTime(EN_EVENT_NO.Task07, 1);
                    fnCreateEventTime(EN_EVENT_NO.Task08, 1);
                    fnCreateEventTime(EN_EVENT_NO.Task09, 1);
                    inReturn = On_Init();               //初期処理
                    break;
                case CRTC.STS_PRO:
                    // タスクのイベントを発行する
                    fnRunEvent(EN_EVENT_NO.Task01);
                    fnRunEvent(EN_EVENT_NO.Task02);
                    fnRunEvent(EN_EVENT_NO.Task03);
                    fnRunEvent(EN_EVENT_NO.Task04);
                    fnRunEvent(EN_EVENT_NO.Task05);
                    fnRunEvent(EN_EVENT_NO.Task06);
                    fnRunEvent(EN_EVENT_NO.Task07);
                    fnRunEvent(EN_EVENT_NO.Task08);
                    fnRunEvent(EN_EVENT_NO.Task09);
                    inReturn = On_Process();            //通常処理
                    break;
                case CRTC.STS_END:
                    inReturn = On_End();                //終了処理
                    break;
                case CRTC.STS_ERR:
                    inReturn = On_Err();                //エラー処理
                    break;
            }

            return inReturn;                            // FUの実行結果を返却
        }
    }
}
